package org.rzo.yacron4j;

import java.text.ParseException;
import java.util.Date;
import java.util.TimeZone;

import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;
import org.quartzx.CronExpression;

public class DSTCronExpression extends CronExpression
{
	final CronExpression _orgCronExpression;
	final TaskOptions _options;

	public DSTCronExpression(String cronExpression, TaskOptions options)
			throws ParseException
	{
		super(cronExpression);
		super.setTimeZone(TimeZone.getTimeZone("UTC"));
		_orgCronExpression = new CronExpression(cronExpression);
		_options = options;
	}

	@Override
	public Date getTimeAfter(Date date)
	{

		Date date1 = super.getTimeAfter(new DateTime(date)
				.withZoneRetainFields(DateTimeZone.UTC).toDate());
		date1 = new DateTime(date1).withZoneRetainFields(
				DateTimeZone.forTimeZone(_options.getTimeZone())).toDate();
		if ((_options.getTimeZone().inDaylightTime(date1) && !(_options
				.getTimeZone().inDaylightTime(date))))
		{
			DateTimeZone dtz = DateTimeZone.forTimeZone(_options.getTimeZone());
			Date dstEnd = new Date(dtz.nextTransition(date.getTime()));
			int dstEndHour = dstEnd.getHours();
			int dstDuration = (dtz.getOffset(date1.getTime()) - dtz
					.getStandardOffset(date1.getTime())) / (60 * 60 * 1000);
			int hour = date1.getHours() + date1.getTimezoneOffset() / 60;
			if (hour < dstEndHour && hour >= dstEndHour - dstDuration)
			{
				if (_options.getDstConflictResolution() == 0)
					return dstEnd;
				else if (_options.getDstConflictResolution() < 0)
					return new Date(date1.getTime() - dstDuration * 60 * 60
							* 1000);
				else
					return date1;
			}
			else
				return _orgCronExpression.getTimeAfter(date);
		}
		else
			return _orgCronExpression.getTimeAfter(date);
	}

	@Override
	public void setTimeZone(TimeZone timeZone)
	{
		_orgCronExpression.setTimeZone(timeZone);
	}

}
